<?php

/**
 * @file
 * Contains \Drupal\miniorange_saml\Form\MiniorangeSamlCustomerSetup.
 */

namespace Drupal\miniorange_saml_idp\Form;

use Drupal\Core\Form\FormBase;
use Drupal\miniorange_saml_idp\Utilities;
use Drupal\miniorange_saml_idp\MiniorangeSAMLIdpConstants;
use Drupal\miniorange_saml_idp\MiniorangeSAMLCustomer;
use Symfony\Component\HttpFoundation\RedirectResponse;

class MiniorangeSamlCustomerSetup extends FormBase
{

    public function getFormId()
    {
        return 'miniorange_saml_customer_setup';
    }

    public function buildForm(array $form, \Drupal\Core\Form\FormStateInterface $form_state)
    {
        $current_status = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_status');

        Utilities::visual_tour_start($form, $form_state);

        if ($current_status == 'VALIDATE_OTP') {

            Utilities::visual_tour_start($form, $form_state);

            $form['markup_top'] = array(
                '#markup' => '<div class="mo_saml_table_layout_1"><div id="Register_Section" class="mo_saml_table_layout mo_saml_container">'
            );

            $form['miniorange_saml_customer_otp_token'] = array(
                '#type' => 'textfield',
                '#title' => t('OTP<span style="color: red">*</span>'),
                '#attributes' => array('style' => 'width:50%'),
                '#suffix' => '<br/>',
            );

            $form['miniorange_saml_customer_validate_otp_button'] = array(
                '#type' => 'submit',
                '#value' => t('Validate OTP'),
                '#submit' => array('::miniorange_saml_idp_validate_otp_submit'),
            );

            $form['miniorange_saml_customer_setup_resendotp'] = array(
                '#type' => 'submit',
                '#value' => t('Resend OTP'),
                '#submit' => array('::miniorange_saml_idp_resend_otp'),
            );

            $form['miniorange_saml_customer_setup_back'] = array(
                '#type' => 'submit',
                '#value' => t('Back'),
                '#submit' => array('::miniorange_saml_idp_back'),
                '#suffix' => '<br><br><br><br><br><br><br><br>
                        <br><br><br><br><br><br></div>',
            );

            Utilities::spConfigGuide($form, $form_state);
            Utilities::AddSupportButton($form, $form_state);

            return $form;
        } elseif ($current_status == 'PLUGIN_CONFIGURATION') {
            $form['markup_top_message'] = array(
                '#markup' => '<div class="mo_saml_table_layout_1"><div class="mo_saml_table_layout mo_saml_container">
                        <div class="mo_saml_welcome_message">Thank you for registering with miniOrange</div><h4>Your Profile: </h4>'
            );

            $header = array(
                'email' => array(
                    'data' => t('Customer Email')
                ),
                'customerid' => array(
                    'data' => t('Customer ID')
                ),
                'token' => array(
                    'data' => t('Token Key')
                ),
                'apikey' => array(
                    'data' => t('API Key')
                ),
            );

            global $base_url;
            $options = [];

            $options[0] = array(
                'email' => \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_email'),
                'customerid' => \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_id'),
                'token' => \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_token'),
                'apikey' => \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_api_key'),
            );

            $form['fieldset']['customerinfo'] = array(
                '#theme' => 'table',
                '#header' => $header,
                '#rows' => $options,
                '#suffix' => '<br><br><br>',
            );

            $form['miniorange_saml_idp_customer_Remove_Account_info'] = array(
                '#markup' => '<br><br><br/><h4>Remove Account: <a href="' . $base_url . MiniorangeSAMLIdpConstants::LICENSE_PAGE_URL . '"> [Premium]</a></h4><p>This section will help you to remove your current logged in account without losing your current configurations.</p>'
            );
            $form['miniorange_saml_idp_customer_Remove_Account'] = array(
                '#type' => 'submit',
                '#value' => t('Remove Account'),
                '#submit' => array('::miniorange_saml_remove_account'),
                '#disabled' => true,
                '#suffix' => '<br><br></div>',
            );

            Utilities::spConfigGuide($form, $form_state);
            Utilities::AddSupportButton($form, $form_state);

            return $form;
        }

        Utilities::visual_tour_start($form, $form_state);

        $form['markup_top'] = array(
            '#markup' => '<div class="mo_saml_table_layout_1"><div id="Register_Section" class="mo_saml_table_layout mo_saml_container">'
        );

        $form['markup_msg_1'] = array(
            '#markup' => '<h2>Register/Login with miniOrange (Optional)</h2><hr><br/>
                        <div class="mo_saml_highlight_background_note"><h3>Why should I register?</h3>You should register so that in case you need help, we can help you with step by step instructions. 
                <b>You will also need a miniOrange account to upgrade to the premium version of the module.</b> 
                We do not store any information except the email that you will use to register with us.</div><br>
                <div class = "mo_saml_highlight_background_note">Please enter a valid email id that you have access to. We will send OTP to this email for verification.</div>'
        );

        $form['miniorange_saml_customer_setup_username'] = array(
            '#type' => 'email',
            '#title' => t('Email<span style="color: red">*</span>'),
            '#description' => t('<b>Note:</b> Use valid EmailId. ( We discourage the use of disposable emails )'),
        );

        $form['miniorange_saml_customer_setup_phone'] = array(
            '#type' => 'textfield',
            '#title' => t('Phone'),
            '#description' => t('<b>Note:</b> We will only call if you need support.'),
        );

        $form['miniorange_saml_customer_setup_password'] = array(
            '#type' => 'password_confirm',
            '#size' => 6,
        );

        $form['miniorange_saml_customer_setup_button'] = array(
            '#type' => 'submit',
            '#value' => t('Submit'),
            '#prefix' => '<br>',
            '#suffix' => '<br><br></div>',
        );

        Utilities::spConfigGuide($form, $form_state);
        Utilities::AddSupportButton($form, $form_state);

        return $form;
    }

    public function submitForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state)
    {
        global $base_url;
        $username = $form['miniorange_saml_customer_setup_username']['#value'];
        $phone = $form['miniorange_saml_customer_setup_phone']['#value'];
        $password = $form['miniorange_saml_customer_setup_password']['#value']['pass1'];
        if (empty($username) || empty($password)) {
            \Drupal::messenger()->addMessage(t('The <b><u>Email Address</u></b> and <b><u>Password</u></b> fields are mandatory.'), 'error');
            return;
        }
        $customer_config = new MiniorangeSAMLCustomer($username, $phone, $password, NULL);
        $check_customer_response = json_decode($customer_config->checkCustomer());
        if ($check_customer_response->status == 'CUSTOMER_NOT_FOUND') {

            \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_customer_admin_email', $username)->save();
            \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_customer_admin_phone', $phone)->save();
            \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_customer_admin_password', $password)->save();
            $send_otp_response = json_decode($customer_config->sendOtp());

            if ($send_otp_response->status == 'SUCCESS') {
                \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_tx_id', $send_otp_response->txId)->save();
                \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_status', 'VALIDATE_OTP')->save();
                \Drupal::messenger()->addMessage(t('Verify email address by entering the passcode sent to @username', [
                    '@username' => $username
                ]));
            }
        } elseif ($check_customer_response->status == 'CURL_ERROR') {
            \Drupal::messenger()->addMessage(t('cURL is not enabled. Please enable cURL'), 'error');
            return;
        } else {
            $customer_keys_response = json_decode($customer_config->getCustomerKeys());

            if (json_last_error() == JSON_ERROR_NONE) {
                \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_customer_id', $customer_keys_response->id)->save();
                \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_customer_admin_token', $customer_keys_response->token)->save();
                \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_customer_admin_email', $username)->save();
                \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_customer_admin_phone', $phone)->save();
                \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_customer_api_key', $customer_keys_response->apiKey)->save();
                \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_status', 'PLUGIN_CONFIGURATION')->save();

                $redirect_url = $base_url . MiniorangeSAMLIdpConstants::LICENSE_PAGE_URL;
                $response = new RedirectResponse($redirect_url);
                $response->send();
            } else {
                \Drupal::messenger()->addMessage(t('Invalid credentials'), 'error');
                return;
            }
        }
    }

    public function miniorange_saml_idp_back(&$form, $form_state)
    {
        \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_status', 'CUSTOMER_SETUP')->save();
        \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->clear('miniorange_miniorange_saml_customer_admin_email')->save();
        \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->clear('miniorange_saml_customer_admin_phone')->save();
        \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->clear('miniorange_saml_tx_id')->save();
        \Drupal::messenger()->addMessage(t('Register/Login with your miniOrange Account'), 'status');
    }

    public function miniorange_saml_idp_resend_otp(&$form, $form_state)
    {
        \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->clear('miniorange_saml_tx_id')->save();
        $username = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_email');
        $phone = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_phone');
        $customer_config = new MiniorangeSAMLCustomer($username, $phone, NULL, NULL);
        $send_otp_response = json_decode($customer_config->sendOtp());
        if ($send_otp_response->status == 'SUCCESS') {
            // Store txID.
            \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_tx_id', $send_otp_response->txId)->save();
            \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_status', 'VALIDATE_OTP')->save();
            \Drupal::messenger()->addMessage(t('Verify email address by entering the passcode sent to @username', array('@username' => $username)));
        }
    }

    public function miniorange_saml_idp_validate_otp_submit(&$form, $form_state)
    {
        global $base_url;
        $otp_token = $form['miniorange_saml_customer_otp_token']['#value'];
        $username = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_email');
        $phone = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_phone');
        $tx_id = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_tx_id');
        $customer_config = new MiniorangeSAMLCustomer($username, $phone, NULL, $otp_token);
        $validate_otp_response = json_decode($customer_config->validateOtp($tx_id));

        if ($validate_otp_response->status == 'SUCCESS') {
            \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->clear('miniorange_saml_tx_id')->save();
            $password = \Drupal::config('miniorange_saml_idp.settings')->get('miniorange_saml_customer_admin_password');
            $customer_config = new MiniorangeSAMLCustomer($username, $phone, $password, NULL);
            $create_customer_response = json_decode($customer_config->createCustomer());
            if ($create_customer_response->status == 'SUCCESS') {
                \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_status', 'PLUGIN_CONFIGURATION')->save();
                \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_customer_admin_email', $username)->save();
                \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_customer_admin_phone', $phone)->save();
                \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_customer_admin_token', $create_customer_response->token)->save();
                \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_customer_id', $create_customer_response->id)->save();
                \Drupal::configFactory()->getEditable('miniorange_saml_idp.settings')->set('miniorange_saml_customer_api_key', $create_customer_response->apiKey)->save();
                \Drupal::messenger()->addMessage(t('Customer account created.'));
            } else if (trim($create_customer_response->message) == 'Email is not enterprise email.') {
                \Drupal::messenger()->addMessage(t('There was an error creating an account for you.<br> You may have entered an invalid Email-Id
            <strong>(We discourage the use of disposable emails) </strong>
            <br>Please try again with a valid email.'), 'error');
                return;
            } else {
                \Drupal::messenger()->addMessage(t('Error creating customer'), 'error');
                return;
            }
        } else {
            \Drupal::messenger()->addMessage(t('Error validating OTP'), 'error');
            return;
        }
    }
}
